package com.agiledeveloper.ticktacktoe;

/**
 * User: Venkat Subramaniam
 * Date: Jul 11, 2004
 * Time: 8:57:39 AM
 */
public class TickTackToeBoard
{
    private String nextPlayerPeg;
    private String[][] pegs = {{"", "", ""}, {"", "", ""}, {"", "", ""}};
    private boolean gameHasBegun;

    public boolean isGameOver()
    {
        return CheckRowMatch() || checkColumnMatch() || checkDiagonalMatch();
    }

    private boolean checkDiagonalMatch()
    {
        boolean result = false;

        if (pegs[0][0].equals(pegs[1][1]) && pegs[1][1].equals(pegs[2][2]) &&
                !pegs[2][2].equals(""))
            result = true;
        if (pegs[2][0].equals(pegs[1][1]) && pegs[1][1].equals(pegs[0][2]) &&
                !pegs[0][2].equals(""))
            result = true;

        return result;
    }

    private boolean CheckRowMatch()
    {
        boolean result = false;

        for (int i = 0; i < 3; i++)
        {
            if(pegs[i][0].equals(pegs[i][1]) &&
                    pegs[i][1].equals(pegs[i][2]) &&
                    !pegs[i][2].equals(""))
            {
                result = true;
            }
        }
        return result;
    }

    private boolean checkColumnMatch()
    {
        boolean result = false;

        for (int i = 0; i < 3; i++)
        {
            if(pegs[0][i].equals(pegs[1][i]) &&
                    pegs[1][i].equals(pegs[2][i]) &&
                    !pegs[2][i].equals(""))
            {
                result = true;
            }
        }
        return result;
    }

    public void firstPlayerIsX(boolean isX)
    {
        if (gameHasBegun)
            throw new TickTackToeBoardException("Game has begun");

        if(isX)
        {
            nextPlayerPeg = "X";
        }
        else
        {
            nextPlayerPeg = "O";
        }
    }

    public boolean isNextPlayerPegX()
    {
        if(nextPlayerPeg == null)
        {
            throw new TickTackToeBoardException("Set first player first");
        }
        boolean result = false;
        if(nextPlayerPeg.equals("X"))
            result = true;

        return result;
    }

    public void placePeg(int row, int column)
    {
        if(isGameOver())
            throw new TickTackToeBoardException("Game over, please give up!");

        if(nextPlayerPeg == null)
            throw new TickTackToeBoardException("Set first player first");

        checkRange(row, column);

        if(!pegs[row][column].equals(""))
        {
            throw new TickTackToeBoardException("Position not empty");
        }

        pegs[row][column] = nextPlayerPeg;

        changePlayer();
        gameHasBegun = true;
    }

    private void changePlayer()
    {
        if (nextPlayerPeg.equals("X"))
            nextPlayerPeg = "O";
        else
            nextPlayerPeg = "X";
    }

    private void checkRange(int row, int column)
    {
        if (row < 0 || row > 2)
            throw new TickTackToeBoardException("Row " + row + " out of range!");

        if (column < 0 || column > 2)
            throw new TickTackToeBoardException("Column " + column + " out of range!");
    }

    public boolean isPegXAt(int row, int column)
    {
        checkRange(row, column);

        boolean result = false;
        if(pegs[row][column].equals("X"))
            result = true;

        return result;
    }

}